/** 
 *   DGPS - Rutinas para la interface serie.
 *
 *   (c) Copyright 2007 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/


#define _GNU_SOURCE

#include <string.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <sys/file.h>
#include <my_stdio.h>


static ssize_t soft_read_time_out(struct timeval tv, int fd, void *buf,
                                  size_t count)
{
        fd_set rfds;
        int fd_max = fd + 1;
        int retval, l = 0;
        struct timeval tvl;

        while ( l < count) {
          memcpy(&tvl, &tv, sizeof(struct timeval));
                FD_ZERO(&rfds);
                FD_SET(fd, &rfds);
                retval = select(fd_max, &rfds, NULL, NULL, &tvl);
                if (retval) {
                        if( FD_ISSET(fd, &rfds) ) {
                            retval = read(fd, buf+l, count - l);
                                l += retval;
                        }
                }else{
			PDBG("time out reading %d/%d.", l, count);
                        return -1;
                }
        }

#if 0
#ifdef VERBOSE_MODE
        if (verbose_flag & PINFO) {
                int __i;
                //fprintf(stderr, __FILE__ " read <- ");
                for(__i=0;__i<l;__i++)
			fprintf(stderr, "%02x ",
				*(((u_char *)buf)+__i) );
                //fprintf(stderr, "\n");
        }
#endif
#endif
        return l;
}



int hw_open(char *name)
{
        int fd;
        struct termios tio;

        /* Open Serial Port : 8 bits data, 1 bit stop, no parity, 9600
           baud */
        if ((fd = open(name, O_RDWR | O_NOCTTY)) < 0 ){
                SYSLOG(LOG_ERR, "Cannot open serial port (%s).", name);
                return -1;
        }

        memset(&tio, 0, sizeof(tio));
        tio.c_cflag &= ~CRTSCTS ;
        /* Zero out the baud rate portion of c_cflag */
        tio.c_cflag &= ~CBAUD;
        /* Set baud rate to 9600 */
        tio.c_cflag |= B9600;
        tio.c_cflag &= ~(CSIZE|PARENB);
        tio.c_cflag |= CS8 ;
        //tio.c_cflag |= PARENB ;
        /* Ignore modem control lines */
        tio.c_cflag |= CLOCAL;
        /* Enable receiver */
        tio.c_cflag |= CREAD;

        /* Turn of Canonical Processing and echo */
        tio.c_lflag &= ~ICANON;
        tio.c_lflag &= ~ECHO;
        /* Set MIN to one and TIME to zero */
        tio.c_cc[VMIN]  = 1;
        tio.c_cc[VTIME] = 0;

        /* Ignore BREAK condition on input */
        tio.c_iflag |= IGNBRK;
        /* ignore framing errors and parity errors */
        tio.c_iflag |=IGNPAR;
        /* Disable NL to CR mapping */
        tio.c_iflag &= ~ICRNL;
        /* Disable XON/XOFF flow control on output and input */
        tio.c_iflag &= ~(IXON|IXOFF);

        /* now clean the modem line and activate the settings for modem */
        tcflush(fd, TCOFLUSH|TCIFLUSH);
        tcsetattr(fd, TCSANOW, &tio);

        return fd;
}


int hw_close(int fd)
{
        return close(fd);
}


int hw_read(int fd, void *buff, size_t len)
{
	struct timeval tv = {
		.tv_sec = 2,
		.tv_usec = 0
	};

	return 	soft_read_time_out(tv, fd, buff, len);
}


int hw_write(int fd, void *buff, size_t count)
{
	fd_set wfds;
	struct timeval tv;
	int retval;
	FD_ZERO(&wfds);
	FD_SET(fd, &wfds);
	tv.tv_sec = 0;
	tv.tv_usec = 20000;

	retval = select(fd+1, NULL, &wfds, NULL, &tv);
	if (retval && FD_ISSET(fd, &wfds)) {
		retval = write(fd, buff, count);
#ifdef VERBOSE_MODE
		if (verbose_flag & PINFO) {
			int __i;
			fprintf(stderr, __FILE__ " write -> ");
			for(__i=0;__i<count;__i++)
				fprintf(stderr, "%02x ",
					*(((u_char *)buff)+__i) );
			fprintf(stderr, "\n");
		}
#endif
	
	} else {
		PVERB(PERR, "time out.");
	}

	return retval;
}

